/* 
 * 1wire.c
 *
 * sample 1wire application
 */

#include <bcm2835.h>
#include <stdio.h>

//uint8_t BUS = RPI_V2_GPIO_P1_07;
uint8_t BUS = RPI_V2_GPIO_P1_03;

#define READ_ROM 		0x33	// 1W Commands
#define SKIP_ROM		0xCC
#define SEARCH_ROM		0xF0

#define CONVERT_T		0x44	// DS1820 commands
#define READ_SCRATCHPAD		0xBE
#define WRIRE_SCRATCHPAD	0x4E


static unsigned char w1_crc8_table[] = {
        0, 94, 188, 226, 97, 63, 221, 131, 194, 156, 126, 32, 163, 253, 31, 65,
        157, 195, 33, 127, 252, 162, 64, 30, 95, 1, 227, 189, 62, 96, 130, 220,
        35, 125, 159, 193, 66, 28, 254, 160, 225, 191, 93, 3, 128, 222, 60, 98,
        190, 224, 2, 92, 223, 129, 99, 61, 124, 34, 192, 158, 29, 67, 161, 255,
        70, 24, 250, 164, 39, 121, 155, 197, 132, 218, 56, 102, 229, 187, 89, 7,
        219, 133, 103, 57, 186, 228, 6, 88, 25, 71, 165, 251, 120, 38, 196, 154,
        101, 59, 217, 135, 4, 90, 184, 230, 167, 249, 27, 69, 198, 152, 122, 36,
        248, 166, 68, 26, 153, 199, 37, 123, 58, 100, 134, 216, 91, 5, 231, 185,
        140, 210, 48, 110, 237, 179, 81, 15, 78, 16, 242, 172, 47, 113, 147,
        205, 17, 79, 173, 243, 112, 46, 204, 146, 211, 141, 111, 49, 178,
        236, 14, 80, 175, 241, 19, 77, 206, 144, 114, 44, 109, 51, 209, 143,
        12, 82, 176, 238, 50, 108, 142, 208, 83, 13, 239, 177, 240, 174, 76,
        18, 145, 207, 45, 115, 202, 148, 118, 40, 171, 245, 23, 73, 8, 86,
        180, 234, 105, 55, 213, 139, 87, 9, 235, 181, 54, 104, 138, 212,
        149, 203, 41, 119, 244, 170, 72, 22, 233, 183, 85, 11, 136, 214, 52,
        106, 43, 117, 151, 201, 74, 20, 246, 168, 116, 42, 200, 150, 21, 75,
        169, 247, 182, 232, 10, 84, 215, 137, 107, 53
};


unsigned char onewire_reset()
{
  uint8_t ret;

  bcm2835_gpio_fsel(BUS, BCM2835_GPIO_FSEL_OUTP);  // bus write
  bcm2835_gpio_write(BUS, 0);                      // bus low
  bcm2835_delayMicroseconds(480);                  // wait 480 us
  bcm2835_gpio_fsel(BUS, BCM2835_GPIO_FSEL_INPT);  // bus read
  bcm2835_delayMicroseconds(70);                   // wait 70 us
  ret = bcm2835_gpio_lev(BUS);                     // detect presence pulse low
                                                   // 60 - 240 us 
  bcm2835_delayMicroseconds(480-70);               // wait after presence pulse
  if (bcm2835_gpio_lev(BUS) == 0)                  // short circuit
    ret = 1;
  return ret;
}

unsigned char onewire_bit_io(unsigned char b)
{
  bcm2835_gpio_fsel(BUS, BCM2835_GPIO_FSEL_OUTP);  // bus write
  bcm2835_gpio_write(BUS, 0);                      // bus low
  bcm2835_delayMicroseconds(1);                    // wait
  if (b)
    bcm2835_gpio_fsel(BUS, BCM2835_GPIO_FSEL_INPT);  // bus read
  bcm2835_delayMicroseconds(15-1);
  if (bcm2835_gpio_lev(BUS) == 0)
    b = 0;
  bcm2835_delayMicroseconds(60-15);
  bcm2835_gpio_fsel(BUS, BCM2835_GPIO_FSEL_INPT);  // bus read
  return b;
}

unsigned int onewire_byte_write(unsigned char b)
{
  unsigned char i = 8, j;
  do
  {
    j = onewire_bit_io(b & 1);
    b >>= 1;
    if (j)
      b |= 0x80;
  } while (--i);
  return b;
}

unsigned int onewire_byte_read()
{
  return onewire_byte_write(0xFF);
}

unsigned char w1_calc_crc8(unsigned char * data, int len)
{
  unsigned char crc = 0;

  while (len--)
    crc = w1_crc8_table[crc ^ *data++];
  return crc;
}



int main(int argc, char **argv)
{

  unsigned char x[9];
  int ret = 0;
  float t;

  if (!bcm2835_init())
    return 1;

  if (onewire_reset())
    printf("onewire_reset: no device found\n");

  onewire_byte_write(READ_ROM);
  for (int i=0; i<8; i++) 
  {
    x[i] = onewire_byte_read();
    printf("%02x ", x[i]);
  }

  if (x[7] == w1_calc_crc8(x, 7))
  {
    printf(" crc o.k.\n");
  } else {
    printf(" crc error.\n");
    ret++;
  }
 
  //printf("calculated crc: %02x\n", w1_calc_crc8(x, 7));

  // initiates a single temperature conversion
  onewire_reset();
  onewire_byte_write(SKIP_ROM);
  onewire_byte_write(CONVERT_T);
  bcm2835_gpio_fsel(BUS, BCM2835_GPIO_FSEL_OUTP);	// parasite power on
  bcm2835_gpio_write(BUS, 1);

  // wait for conversion
  bcm2835_delay(1000);
  onewire_reset();
  onewire_byte_write(SKIP_ROM);
  onewire_byte_write(READ_SCRATCHPAD);

  for (int i=0; i<9; i++)
  {
    x[i] = onewire_byte_read();
    printf("%02x ", x[i]);
  }

  if (x[8] == w1_calc_crc8(x, 8))
  {
    printf(" crc o.k.\n");
    // calculate temperature
    //t = (float)(x[0] | x[1] << 8)/2;  // 9bit resolution
    t = (float)(x[0] >> 1 | x[1] << 7);  // resolution grater than 9 bit
    t = t - 0.25 + (float)(((float)x[7]-(float)x[6])/(float)x[7]);
    printf(" Temperature: %.3f °C\n", t);
  } else {
    printf(" crc error.\n");
    ret++;
  }
  
  bcm2835_close();
  return ret;
}
