/* 
 * fs20.c
 *
 * sample fs20 sender application
 */

#include <bcm2835.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <strings.h>

uint8_t DATA = RPI_V2_GPIO_P1_03;

#define HI8(x) ((uint8_t)((x) >> 8))
#define LO8(x) ((uint8_t)(x))

uint8_t parity_even_bit(uint8_t v)
{
  uint8_t parity = 0;

  while (v)
  {
    parity = !parity;
    v = v & (v - 1);
  }
  return (parity);
}

void fs20_init(void)
{
  // init the DATA out port  
  bcm2835_gpio_fsel(DATA, BCM2835_GPIO_FSEL_OUTP);
}

void fs20_send_zero(void)
{
  bcm2835_gpio_write(DATA, 1);
  bcm2835_delayMicroseconds(375);
  bcm2835_gpio_write(DATA, 0);
  bcm2835_delayMicroseconds(395);
}

void fs20_send_one(void)
{
  bcm2835_gpio_write(DATA, 1);
  bcm2835_delayMicroseconds(595);
  bcm2835_gpio_write(DATA, 0);
  bcm2835_delayMicroseconds(630);
}

void fs20_send_sync(void)
{
  for (uint8_t i = 0; i < 12; i++)
    fs20_send_zero();
  fs20_send_one();
}

void fs20_send_bit(uint8_t bit)
{
  if (bit > 0) 
    fs20_send_one();
  else 
    fs20_send_zero();
}

void fs20_send_byte(uint8_t byte)
{
  uint8_t i = 7;

  do {
    fs20_send_bit(byte & (1<<i));
  } while (i-- > 0);

  fs20_send_bit(parity_even_bit(byte));
}

void fs20_send(uint16_t housecode, uint8_t address, uint8_t command)
{
  
  for (uint8_t i = 0; i < 3; i++) {
    fs20_send_sync();

    uint8_t sum = 6; /* magic constant 6 from fs20 protocol definition... */

    fs20_send_byte(HI8(housecode));
    sum += HI8(housecode);
    fs20_send_byte(LO8(housecode));
    sum += LO8(housecode);
    fs20_send_byte(address);
    sum += address;
    fs20_send_byte(command);
    sum += command;
    fs20_send_byte(sum);

    fs20_send_zero();

    bcm2835_delay(10); // delay 10ms
  }
}

void usage(char *prog)
{
  printf("usage %s [-h] [-v] [-a address] -c command\n", prog);
  printf("          -h           show this help\n");
  printf("          -v           more verbose\n");
  printf("          -a address   device or group address 0..256 (default 1)\n");
  printf("          -c command   command\n");
  printf("             on        switch device on\n");
  printf("             off       switch device off\n");
  printf("             toggle    toggle state\n");
  printf("             up        dim up\n");
  printf("             down      dim down\n");
  printf("\n");
}

int main(int argc, char **argv)
{
  int ret = 0;
  int c;
  int addr = 0;
  int cmd;

  while ((c = getopt (argc, argv, "a:c:vh")) != -1)
    switch (c)
    {
      case 'v':
        printf("verbose\n");
        break;
      case 'a':
         addr = strtol(optarg, 0, 10);
         printf("device: %02x\n", addr);
         break;
      case 'c':
         printf("command: %s\n", optarg);
         if (!strcasecmp(optarg, "off")) {
             printf("OFF\n");
             cmd = 0x00;
         } else if (!strcasecmp(optarg, "on")) {
             printf("ON\n");
             cmd = 0x10;
         } else if (!strcasecmp(optarg, "toggle")) {
             printf("TOGGLE\n");
             cmd = 0x12;
         } else if (!strcasecmp(optarg, "up")) {
             printf("DIM UP\n");
             cmd = 0x13;
         } else if (!strcasecmp(optarg, "down")) {
             printf("DIM_DOWN\n");
             cmd = 0x14;
         } else {
             usage(argv[0]);
             return (-1);
         }
         break;
      default:
         usage(argv[0]);
         return (-1);
    }

  if (!bcm2835_init())
    return 1;

  printf ("fs20_init\n");
  fs20_init();

  printf ("fs20_send: addr 0x%02x, cmd 0x%02x\n", addr, cmd); 
  fs20_send(0x852f,addr,cmd);

/*
  printf ("fs20_send - on\n"); 
  fs20_send(0x852f,0x01,0x11); // 6+  133+47+1+0 = 187
  bcm2835_delay(5000); 
  printf ("fs20_send - off\n"); 
  fs20_send(0x852f,0x01,0x00); // 6+  133+47+1+0 = 187
*/
  
  bcm2835_close();
  return ret;
}
