/* 
 * fs20.c
 *
 * sample fs20 sender application
 * with rfm12 transmitter module
 */

#include <bcm2835.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <strings.h>

#define HOUSECODE       0x493A
// #define HOUSECODE       0x852f

#define HI8(x) ((uint8_t)((x) >> 8))
#define LO8(x) ((uint8_t)(x))

// RFM12 commands
#define RF_XMITTER_ON   0x823D
#define RF_IDLE_MODE    0x820D


uint8_t parity_even_bit(uint8_t v)
{
  uint8_t parity = 0;

  while (v)
  {
    parity = !parity;
    v = v & (v - 1);
  }
  return (parity);
}

void spi_xfer(uint16_t cmd)
{
  char buf[2];
  buf[0] = cmd >> 8;
  buf[1] = cmd;
  //printf("Send to SPI:   0x%04x\n", cmd);
  bcm2835_spi_transfern(buf, sizeof(buf));
  //printf("Read from SPI: 0x%04x\n", buf[0]<<8| buf[1]);
}

void spi_init(void)
{
  // init the spi
  bcm2835_spi_setBitOrder(BCM2835_SPI_BIT_ORDER_MSBFIRST);        // The default
  bcm2835_spi_setDataMode(BCM2835_SPI_MODE0);                     // The default
  bcm2835_spi_setClockDivider(BCM2835_SPI_CLOCK_DIVIDER_256);     // ca. 1MHz
  bcm2835_spi_chipSelect(BCM2835_SPI_CS0);                        // The default
  bcm2835_spi_setChipSelectPolarity(BCM2835_SPI_CS0, LOW);        // the default
}

void rfm12_init(void)
{
  spi_xfer(0x80C7 | (2 << 4)); // EL,EF,12.0pF
  spi_xfer(0x8209);               // enable xtal, disable clk pin
  spi_xfer(0xA68a);               //(0xa68a FS20 868.250)  90khz )
  spi_xfer(0xC606);               // c606 57.6Kbps (38.4: 8, 19.2: 11, 9.6: 23, 4.8: 47)
  spi_xfer(0x94A0);               // VDI,FAST,134kHz,0dBm,-103dBm
  spi_xfer(0xC2AC);               // data filter
  spi_xfer(0xCA83);               // FIFO8,SYNC,!ff,DR
  spi_xfer(0xCE00 | 0);       // sync byte
  spi_xfer(0xC483);               // @PWR,NO RSTRIC,!st,!fi,OE,EN
  spi_xfer(0x9850);               // !mp,90kHz Devation
  spi_xfer(0xCC77);               // pll settings
  spi_xfer(0xE000);               // NOT USE
  spi_xfer(0xC800);               // NOT USE
  spi_xfer(0xC040);               // 1.66MHz,2.2V
}

void fs20_send_zero(void)
{
  spi_xfer(RF_XMITTER_ON);
  bcm2835_delayMicroseconds(400+165);
  spi_xfer(RF_IDLE_MODE);
  bcm2835_delayMicroseconds(400-200);
}

void fs20_send_one(void)
{
  spi_xfer(RF_XMITTER_ON);
  bcm2835_delayMicroseconds(600+165);
  spi_xfer(RF_IDLE_MODE);
  bcm2835_delayMicroseconds(600-200);
}

void fs20_send_sync(void)
{
  for (uint8_t i = 0; i < 12; i++)
    fs20_send_zero();
  fs20_send_one();
}

void fs20_send_bit(uint8_t bit)
{
  if (bit > 0) 
    fs20_send_one();
  else 
    fs20_send_zero();
}

void fs20_send_byte(uint8_t byte)
{
  uint8_t i = 7;

  do {
    fs20_send_bit(byte & (1<<i));
  } while (i-- > 0);

  fs20_send_bit(parity_even_bit(byte));
}

void fs20_send(uint16_t housecode, uint8_t address, uint8_t command)
{

  // all commands repeated 3 times
  // dim commands repeated only 2 times
  uint8_t repeat=3; 
  if (command >= 0x13 && command <= 0x15) 
    repeat = 2;

  for (uint8_t i = 0; i < repeat; i++) {
    fs20_send_sync();

    uint8_t sum = 6; /* magic constant 6 from fs20 protocol definition... */

    fs20_send_byte(HI8(housecode));
    sum += HI8(housecode);
    fs20_send_byte(LO8(housecode));
    sum += LO8(housecode);
    fs20_send_byte(address);
    sum += address;
    fs20_send_byte(command);
    sum += command;
    fs20_send_byte(sum);

    fs20_send_zero();

    bcm2835_delay(10); // delay 10ms
  }
}

void usage(char *prog)
{
  printf("usage %s [-h] [-v] [-a address] -c command\n", prog);
  printf("          -h           show this help\n");
  printf("          -v           more verbose\n");
  printf("          -a address   device or group address 0..256 (default 1)\n");
  printf("          -c command   command\n");
  printf("             on        switch device on\n");
  printf("             off       switch device off\n");
  printf("             toggle    toggle state\n");
  printf("             up        dim up\n");
  printf("             down      dim down\n");
  printf("          -r raw command (hex code)\n");
  printf("\n");
}

int main(int argc, char **argv)
{
  int ret = 0;
  int c;
  int addr = 0;
  int cmd;

  while ((c = getopt (argc, argv, "a:c:r:vh")) != -1)
    switch (c)
    {
      case 'v':
        printf("verbose\n");
        break;
      case 'a':
         addr = strtol(optarg, 0, 10);
         printf("device: %02x\n", addr);
         break;
      case 'c':
         printf("command: %s\n", optarg);
         if (!strcasecmp(optarg, "off")) {
             printf("OFF\n");
             cmd = 0x00;
         } else if (!strcasecmp(optarg, "on")) {
             printf("ON\n");
             cmd = 0x10;
         } else if (!strcasecmp(optarg, "toggle")) {
             printf("TOGGLE\n");
             cmd = 0x12;
         } else if (!strcasecmp(optarg, "up")) {
             printf("DIM UP\n");
             cmd = 0x13;
         } else if (!strcasecmp(optarg, "down")) {
             printf("DIM_DOWN\n");
             cmd = 0x14;
         } else {
             usage(argv[0]);
             return (-1);
         }
         break;
      case 'r':
         cmd = strtol(optarg, 0, 16);
         printf("raw command: %02x\n", cmd);
         break;
      default:
         usage(argv[0]);
         return (-1);
    }

  if (!bcm2835_init())
    return 1;

  printf ("spi_init\n");
  bcm2835_spi_begin();
  spi_init();

  printf ("rfm12_init\n");
  rfm12_init();

  printf ("fs20_send: addr 0x%02x, cmd 0x%02x\n", addr, cmd); 
  fs20_send(HOUSECODE,addr,cmd);

  bcm2835_close();
  return ret;
}
