#!/usr/bin/python
# coding: utf8
#
#

import sys, time, smbus

# bus, address
bus = 1
addr = 0x77
# oversampling settings [0,1,2,3]
oss = 3

def read_bmp085(bus, addr, oss=0, debug=0):
  ret={}
  # init i2c device
  i2c = smbus.SMBus(bus)

  # read calibration data
  calibration = read_calibration_data(i2c, addr)

  # read temperature 
  UT = read_temp(i2c, addr)
  ret['temperature'] = convert_temp(calibration, UT)

  # read pressure
  UP = read_pressure(i2c, addr, oss, debug=0)
  ret['pressure'] = float(convert_pressure(calibration, UP, UT, oss))
  return (ret)

def read_signed(i2c, addr, reg, debug=0):
  try:
    msb = i2c.read_byte_data(addr, reg)
    if (msb & 0x80):
      msb -= 256
    lsb = i2c.read_byte_data(addr, reg+1)
    if (debug):
      print "read: %02x %04x " % (reg, (msb<<8|lsb))
      # print "read: %d " % ((msb<<8|lsb))
    return (msb<<8|lsb)   
  except IOError, err:
    print "Error read register %02x." % reg

def read_unsigned(i2c, addr, reg, debug=0):
  try:
    msb = i2c.read_byte_data(addr, reg)
    lsb = i2c.read_byte_data(addr, reg+1)
    if (debug):
      print "read: %02x %04x " % (reg, (msb<<8|lsb))
      #print "read: %d " % ((msb<<8|lsb))
    return (msb<<8|lsb)   
  except IOError, err:
    print "Error read register %02x." % reg

def write_command(i2c, addr, reg, cmd, debug=0):
  try:
    i2c.write_byte_data(addr, reg, cmd)
    if (debug):
      print "write: %02x to %02x " % (cmd, reg)
  except IOError, err:
    print "Error write to register %02x." % reg

def read_temp(i2c, addr, debug=0):
  # start temp conversation
  write_command(i2c, addr, 0xF4, 0x2E, debug)
  time.sleep(0.005)
  # read temp
  return (read_unsigned(i2c, addr, 0xF6))

def convert_temp(calibration, UT):
  # calculate temp
  X1 = ((UT - calibration['AC6']) * calibration['AC5']) >> 15
  X2 = (calibration['MC'] << 11) / (X1 + calibration['MD'])
  B5 = X1 + X2
  T = ((B5 + 8) >> 4) / 10.0
  return (T)

def read_pressure(i2c, addr, oss=0, debug=0):
  # oss - oversampling rate 
  # start pressure conversation
  try:
    write_command(i2c, addr, 0xF4, 0x34 + (oss<<6), debug)
    # wait for conversion
    if (oss == 0):
      time.sleep(0.005)    # 4.5ms
    elif (oss == 1):
      time.sleep(0.008)    # 7.5ms
    elif (oss == 2):
      time.sleep(0.014)    # 13.5ms
    else:
      time.sleep(0.026)    # 25.5ms
    # read raw pressure
    reg = 0xF6
    msb = i2c.read_byte_data(addr, reg)
    lsb = i2c.read_byte_data(addr, reg+1)
    xlsb = i2c.read_byte_data(addr, reg+2)
    raw_pressure = ((msb << 16) + (lsb << 8) + xlsb) >> (8 - oss)
    if (debug):
      print "read raw pressure: %02x %04x " % (reg, raw_pressure)
    return (raw_pressure)   
  except IOError, err:
    print "Error read register %02x." % reg

def convert_pressure(calibration, UP, UT, oss):
  # calc temp again
  X1 = ((UT - calibration['AC6']) * calibration['AC5']) >> 15
  X2 = (calibration['MC'] << 11) / (X1 + calibration['MD'])
  B5 = X1 + X2
  # calculate pressure
  B6 = B5 - 4000
  X1 = (calibration['B2'] * (B6 * B6) >> 12 ) >> 11
  X2 = (calibration['AC2'] * B6) >> 11
  X3 = X1 + X2
  B3 = (((calibration['AC1'] * 4 + X3) << oss) + 2) / 4
  X1 = (calibration['AC3'] * B6) >> 13
  X2 = (calibration['B1'] * ((B6 * B6) >> 12 )) >> 16
  X3 = ((X1 + X2) + 2) >> 2
  B4 = (calibration['AC4'] * (X3 + 32768)) >> 15
  B7 = (UP - B3) * (50000 >> oss)
  if (B7 < 0x80000000):
    p = (B7 * 2) / B4
  else:
    p = (B7 / B4) * 2
  X1 = (p >> 8) * (p >> 8)
  X1 = (X1 * 3038) >> 16
  X2 = (-7357 * p) >> 16
  p = p + ((X1 + X2 + 3791) >> 4)
  return (p)

def read_calibration_data(i2c, addr, debug=0):
  calibration = {}
  calibration['AC1'] = read_signed(i2c, addr, 0xAA, debug)
  calibration['AC2'] = read_signed(i2c, addr, 0xAC, debug)
  calibration['AC3'] = read_signed(i2c, addr, 0xAE, debug)
  calibration['AC4'] = read_unsigned(i2c, addr, 0xB0, debug)
  calibration['AC5'] = read_unsigned(i2c, addr, 0xB2, debug)
  calibration['AC6'] = read_unsigned(i2c, addr, 0xB4, debug)
  calibration['B1'] = read_signed(i2c, addr, 0xB6, debug)
  calibration['B2'] = read_signed(i2c, addr, 0xB8, debug)
  calibration['MB'] = read_signed(i2c, addr, 0xBA, debug)
  calibration['MC'] = read_signed(i2c, addr, 0xBC, debug)
  calibration['MD'] = read_signed(i2c, addr, 0xBE, debug)
  return calibration


if __name__ == '__main__':
  # altitude
  altitude = 330.0
  debug = 0

  while True:
    # enter main loop
    data = read_bmp085(bus, addr, oss, debug)

    print "Temperatur:              %.1f C" % data['temperature']
    print "Pressure (absolute):    %0.1f hPa" % (data['pressure']/100)

    # pressure at sea level
    p1 = float(data['pressure'])
    po = p1 / pow(1-(altitude/44330), 5.255)
    print "Pressure at sea level: %0.1f hPa" % (po/100)
    print

    time.sleep(5)

